@@ -1,9 +1,10 @@
+import enum
 from datetime import date, datetime
 from functools import partial
 from itertools import filterfalse
 from types import MappingProxyType as MappingProxy
 from typing import (
-    Sequence, Any, Mapping, Dict, TypeVar, Type, Optional, Container, Tuple
+    Sequence, Any, Mapping, Dict, TypeVar, Type, Optional, Container, Tuple,
 )
 from typing_extensions import Final
 
@@ -21,6 +22,8 @@ from AnyQt.QtWidgets import QStyledItemDelegate, QStyleOptionViewItem, \
     QApplication, QStyle
 
 from orangewidget.utils.cache import LRUCache
+from orangewidget.utils import enum_as_int
+
 
 A = TypeVar("A")
 
@@ -44,12 +47,20 @@ class ModelItemCache(QObject):
     {0: ...
 
     """
+    #: The cache key is a tuple of the persistent index of the *parent*,
+    #: row and column. The parent is used because of performance regression in
+    #: Qt6 ~QPersistentModelIndex destructor when there are many (different)
+    #: persistent indices registered with a model. Using parent, row, column
+    #: coalesces these.
+    #: NOTE: QPersistentModelIndex's hash changes when it is invalidated;
+    #: it must be purged from __cache_data before that (see `__connect_helper`)
+    __KEY = Tuple[QPersistentModelIndex, int, int]
     __slots__ = ("__model", "__cache_data")
 
     def __init__(self, *args, maxsize=100 * 200, **kwargs):
         super().__init__(*args, **kwargs)
         self.__model: Optional[QAbstractItemModel] = None
-        self.__cache_data: 'LRUCache[QPersistentModelIndex, Any]' = LRUCache(maxsize)
+        self.__cache_data: 'LRUCache[ModelItemCache.__KEY, Any]' = LRUCache(maxsize)
 
     def __connect_helper(self, model: QAbstractItemModel) -> None:
         model.dataChanged.connect(self.invalidate)
@@ -105,9 +116,7 @@ class ModelItemCache(QObject):
         model = index.model()
         if model is not self.__model:
             self.setModel(model)
-        # NOTE: QPersistentModelIndex's hash changes when it is invalidated;
-        # it must be purged from __cache_data before that (`__connect_helper`)
-        key = QPersistentModelIndex(index)
+        key = QPersistentModelIndex(index.parent()), index.row(), index.column()
         try:
             item = self.__cache_data[key]
         except KeyError:
@@ -126,7 +135,7 @@ class ModelItemCache(QObject):
         model = index.model()
         if model is not self.__model:
             self.setModel(model)
-        key = QPersistentModelIndex(index)
+        key = QPersistentModelIndex(index.parent()), index.row(), index.column()
         try:
             item = self.__cache_data[key]
         except KeyError:
@@ -154,16 +163,16 @@ def cast_(type_: Type[A], value: Any) -> Optional[A]:
 
 # QStyleOptionViewItem.Feature aliases as python int. Feature.__ior__
 # implementation is slower then int.__ior__
-_QStyleOptionViewItem_HasDisplay = int(QStyleOptionViewItem.HasDisplay)
-_QStyleOptionViewItem_HasCheckIndicator = int(QStyleOptionViewItem.HasCheckIndicator)
-_QStyleOptionViewItem_HasDecoration = int(QStyleOptionViewItem.HasDecoration)
+_QStyleOptionViewItem_HasDisplay = enum_as_int(QStyleOptionViewItem.HasDisplay)
+_QStyleOptionViewItem_HasCheckIndicator = enum_as_int(QStyleOptionViewItem.HasCheckIndicator)
+_QStyleOptionViewItem_HasDecoration = enum_as_int(QStyleOptionViewItem.HasDecoration)
 
 
 class _AlignmentFlagsCache(dict):
     # A cached int -> Qt.Alignment cache. Used to avoid temporary Qt.Alignment
     # flags object (de)allocation.
-    def __missing__(self, key: int) -> Qt.Alignment:
-        a = Qt.Alignment(key)
+    def __missing__(self, key: int) -> Qt.AlignmentFlag:
+        a = Qt.AlignmentFlag(key)
         self.setdefault(key, a)
         return a
 
@@ -242,7 +251,7 @@ def init_style_option(
             pix = QPixmap.fromImage(value)
             option.icon = QIcon(value)
             option.decorationSize = (pix.size() / pix.devicePixelRatio()).toSize()
-    option.features |= features
+    option.features |= QStyleOptionViewItem.ViewItemFeature(features)
 
 
 class CachedDataItemDelegate(QStyledItemDelegate):
@@ -352,17 +361,18 @@ class StyledItemDelegate(QStyledItemDelegate):
         return super().displayText(value, locale)
 
 
-_Qt_AlignRight = int(Qt.AlignRight)
-_Qt_AlignLeft = int(Qt.AlignLeft)
-_Qt_AlignHCenter = int(Qt.AlignHCenter)
-_Qt_AlignTop = int(Qt.AlignTop)
-_Qt_AlignBottom = int(Qt.AlignBottom)
-_Qt_AlignVCenter = int(Qt.AlignVCenter)
+_Qt_AlignRight = enum_as_int(Qt.AlignRight)
+_Qt_AlignLeft = enum_as_int(Qt.AlignLeft)
+_Qt_AlignHCenter = enum_as_int(Qt.AlignHCenter)
+_Qt_AlignTop = enum_as_int(Qt.AlignTop)
+_Qt_AlignBottom = enum_as_int(Qt.AlignBottom)
+_Qt_AlignVCenter = enum_as_int(Qt.AlignVCenter)
 
 _StaticTextKey = Tuple[str, QFont, Qt.TextElideMode, int]
 _PenKey = Tuple[str, int]
-_State_Mask = int(QStyle.State_Selected | QStyle.State_Enabled |
-                  QStyle.State_Active)
+_State_Mask = enum_as_int(
+    QStyle.State_Selected | QStyle.State_Enabled | QStyle.State_Active
+)
 
 
 class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
@@ -436,7 +446,7 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
             rect.width()
         )
         tsize = st.size()
-        textalign = int(option.displayAlignment)
+        textalign = enum_as_int(option.displayAlignment)
         text_pos_x = text_pos_y = 0.0
 
         if textalign & _Qt_AlignLeft:
@@ -467,8 +477,7 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
         try:
             return self.__static_text_lru_cache[text, font, elideMode, width]
         except KeyError:
-            text = fontMetrics.elidedText(text, elideMode, width)
-            st = QStaticText(text)
+            st = QStaticText(fontMetrics.elidedText(text, elideMode, width))
             st.prepare(QTransform(), font)
             # take a copy of the font for cache key
             key = text, QFont(font), elideMode, width
@@ -478,18 +487,23 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
     def __pen_cache(self, palette: QPalette, state: QStyle.State) -> QPen:
         """Return a QPen from the `palette` for `state`."""
         # NOTE: This method exists mostly to avoid QPen, QColor (de)allocations.
-        key = palette.cacheKey(), int(state) & _State_Mask
+        key = palette.cacheKey(), enum_as_int(state) & _State_Mask
         try:
             return self.__pen_lru_cache[key]
         except KeyError:
-            cgroup = QPalette.Normal if state & QStyle.State_Active else QPalette.Inactive
-            cgroup = cgroup if state & QStyle.State_Enabled else QPalette.Disabled
-            role = QPalette.HighlightedText if state & QStyle.State_Selected else QPalette.Text
-            pen = QPen(palette.color(cgroup, role))
+            pen = QPen(text_color_for_state(palette, state))
             self.__pen_lru_cache[key] = pen
             return pen
 
 
+def text_color_for_state(palette: QPalette, state: QStyle.State) -> QColor:
+    """Return the appropriate `palette` text color for the `state`."""
+    cgroup = QPalette.Normal if state & QStyle.State_Active else QPalette.Inactive
+    cgroup = cgroup if state & QStyle.State_Enabled else QPalette.Disabled
+    role = QPalette.HighlightedText if state & QStyle.State_Selected else QPalette.Text
+    return palette.color(cgroup, role)
+
+
 class BarItemDataDelegate(DataDelegate):
     """
     An delegate drawing a horizontal bar below its text.
@@ -576,7 +590,7 @@ class BarItemDataDelegate(DataDelegate):
             if color is None:
                 color = self.color
             if not color.isValid():
-                color = opt.palette.color(QPalette.Foreground)
+                color = opt.palette.color(QPalette.WindowText)
             rect = option.rect
             pw = self.penWidth
             hmargin = 3 + pw / 2  # + half pen width for the round line cap
